"""A creation workflow that adds existing office routers to the coreDB."""

import ipaddress

from orchestrator import workflow
from orchestrator.forms import FormPage
from orchestrator.targets import Target
from orchestrator.types import FormGenerator, State, SubscriptionLifecycle
from orchestrator.workflow import StepList, done, init, step
from orchestrator.workflows.steps import resync, set_status, store_process_subscription

from gso.products import ProductName
from gso.products.product_types import office_router
from gso.products.product_types.office_router import OfficeRouterInactive
from gso.services import subscriptions
from gso.services.partners import get_partner_by_name
from gso.services.subscriptions import get_site_by_name
from gso.utils.shared_enums import PortNumber, Vendor


@step("Create subscription")
def create_subscription(partner: str) -> State:
    """Create a new subscription object."""
    partner_id = get_partner_by_name(partner)["partner_id"]
    product_id = subscriptions.get_product_id_by_name(ProductName.OFFICE_ROUTER)
    subscription = OfficeRouterInactive.from_product_id(product_id, partner_id)

    return {
        "subscription": subscription,
        "subscription_id": subscription.subscription_id,
    }


def initial_input_form_generator() -> FormGenerator:
    """Generate a form that is filled in using information passed through the :term:`API` endpoint."""

    class ImportOfficeRouter(FormPage):
        class Config:
            title = "Import an office router"

        partner: str
        office_router_site: str
        office_router_fqdn: str
        office_router_ts_port: PortNumber
        office_router_lo_ipv4_address: ipaddress.IPv4Address
        office_router_lo_ipv6_address: ipaddress.IPv6Address

    user_input = yield ImportOfficeRouter

    return user_input.dict()


@step("Initialize subscription")
def initialize_subscription(
    subscription: OfficeRouterInactive,
    office_router_fqdn: str,
    office_router_ts_port: PortNumber,
    office_router_site: str,
    office_router_lo_ipv4_address: ipaddress.IPv4Address | None = None,
    office_router_lo_ipv6_address: ipaddress.IPv6Address | None = None,
) -> State:
    """Initialise the office router subscription using input data."""
    subscription.office_router.office_router_ts_port = office_router_ts_port
    site_obj = get_site_by_name(office_router_site).site
    subscription.office_router.office_router_site = site_obj
    subscription.office_router.office_router_fqdn = office_router_fqdn
    subscription.description = f"Office router {office_router_fqdn}"
    subscription.office_router.office_router_lo_ipv4_address = office_router_lo_ipv4_address
    subscription.office_router.office_router_lo_ipv6_address = office_router_lo_ipv6_address
    subscription.office_router.vendor = Vendor.JUNIPER

    subscription = office_router.OfficeRouterProvisioning.from_other_lifecycle(
        subscription, SubscriptionLifecycle.PROVISIONING
    )

    return {"subscription": subscription}


@workflow(
    "Import office router",
    initial_input_form=initial_input_form_generator,
    target=Target.CREATE,
)
def import_office_router() -> StepList:
    """Import an office router without provisioning it."""
    return (
        init
        >> create_subscription
        >> store_process_subscription(Target.CREATE)
        >> initialize_subscription
        >> set_status(SubscriptionLifecycle.ACTIVE)
        >> resync
        >> done
    )
